import React, { createContext, useContext } from 'react';
import { useMoveDelegationForm } from '@onomy/react-hub';

import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

export type MoveDelegationContextType = ReturnType<typeof useMoveDelegationForm>;

export const MoveDelegationContext = createContext<MoveDelegationContextType>(null as any);

export function useMoveDelegation() {
  const state = useContext(MoveDelegationContext);
  if (!state) throw new Error('Missing MoveDelegationContext');
  return state;
}

function defaultRenderError(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function MoveDelegationProvider({
  children,
  renderLoading,
  renderError = defaultRenderError,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: () => JSX.Element | JSX.Element[] | null;
  renderError?: typeof defaultRenderError;
} & Parameters<typeof useMoveDelegationForm>[0]) {
  const state = useMoveDelegationForm(props);

  if (renderLoading && (state.loadingStatus.pending || !state.srcDelegation))
    return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);

  return <MoveDelegationContext.Provider value={state}>{children}</MoveDelegationContext.Provider>;
}
